Apache Spark Dataset API in HGrid247-3.5

Apache Spark is an open-source distributed computing framework used to process large-scale data at high speed.

Spark provides: Currently Spark is one of the main components in the Big Data ecosystem along with Apache Hadoop . Spark Architecture

Spark architecture consists of several main components.

    Driver Program
           ↓
    Spark Context
           ↓
    Cluster Manager (YARN / Standalone / Mesos)
           ↓
    Worker Nodes
           ↓
    Executors
These components allow data processing to be performed in parallel on multiple nodes .

Dataset API

Dataset is a distributed data structure introduced in Spark to combine the advantages of: The dataset is:
a distributed data set that is strongly typed and can be processed in parallel.

Characteristics of Dataset

The dataset has several main characteristics:
    Feature                 Description
    ------------------------------------------------------------
    Distributed             Data is spread across many cluster nodes
    Strongly Typed          Have a clear data type
    Optimized Execution     Using Spark Catalyst Optimizer
    Fault Tolerant          Spark can recover data if a node fails.

Dataset vs RDD

Comparison of Dataset with RDD:
    Feature             RDD                 Dataset
    ---------------------------------------------------------
    Type Safety         Not too tight       Strong typing
    Optimization	No optimizer        Catalyst optimizer
    Performance         Fast                More optimal
    API                 Low level           High level
Dataset is a development of RDD with additional optimization from the Spark SQL engine .

Dataset Encoder

Dataset uses Encoder to: The encoder acts as a Spark-specific serializer .

Process:
    Object
       ↓
    Encoder
       ↓
    Binary Format
       ↓
    Distributed Processing
Encoders allow Spark to perform query optimization efficiently.

Dataset Workflow in HGrid247

HGrid247 can automatically generate Spark Dataset API code from workflows created in the GUI.

The output is usually a file:
    projectname_spark2ds.jar

Example Spark Dataset Execution

Example of running a Spark job generated by HGrid247:
    spark-submit --class workshop.Lab01 --master yarn workshop_spark2ds.jar /user/hadoop/input /user/hadoop/output
Explanation:
    Parameter                       Description
    ---------------------------------------------------
    --class workshop.Lab01          the main class being run
    --master yarn                   using YARN cluster
    workshop_spark2ds.jar           Spark application files
    /user/hadoop/input              input folder in HDFS
    /user/hadoop/output             output folder in HDFS

Running in Client Mode

Another alternative is using client deploy mode .
    spark-submit --class workshop.Lab01 --deploy-mode client workshop_spark2ds.jar /user/hadoop/input /user/hadoop/output
Mode differences:
    Mode	Description
    ----------------------------------------
    cluster	drivers running in clusters
    client	driver running on client machine

Advantages of Dataset API

Using Dataset provides several advantages:
  1. Performance Optimization
    The dataset uses Catalyst Optimizer to improve query performance.
  2. Type Safety
    Datasets ensure that data type errors can be detected during compilation.
  3. Integration with Spark SQL
    The dataset can use:
    • SQL query
    • DataFrame operations
    • functional transformations
  4. Efficient Serialization
    Encoders make the process of transferring data between nodes more efficient.

List of options

      --master MASTER_URL         spark://host:port, mesos://host:port, yarn, or local.
      --deploy-mode DEPLOY_MODE   Whether to launch the driver program locally ("client") or
                                  on one of the worker machines inside the cluster ("cluster")
                                  (Default: client).
      --class CLASS_NAME          Your application's main class (for Java / Scala apps).
      --name NAME                 A name of your application.
      --jars JARS                 Comma-separated list of local jars to include on the driver
                                  and executor classpaths.
      --packages                  Comma-separated list of maven coordinates of jars to include
                                  on the driver and executor classpaths. Will search the local
                                  maven repo, then maven central and any additional remote
                                  repositories given by --repositories. The format for the
                                  coordinates should be groupId:artifactId:version.
      --exclude-packages          Comma-separated list of groupId:artifactId, to exclude while
                                  resolving the dependencies provided in --packages to avoid
                                  dependency conflicts.
      --repositories              Comma-separated list of additional remote repositories to
                                  search for the maven coordinates given with --packages.
      --py-files PY_FILES         Comma-separated list of .zip, .egg, or .py files to place
                                  on the PYTHONPATH for Python apps.
      --files FILES               Comma-separated list of files to be placed in the working
                                  directory of each executor.

      --conf PROP=VALUE           Arbitrary Spark configuration property.
      --properties-file FILE      Path to a file from which to load extra properties. If not
                                  specified, this will look for conf/spark-defaults.conf.

      --driver-memory MEM         Memory for driver (e.g. 1000M, 2G) (Default: 1024M).
      --driver-java-options       Extra Java options to pass to the driver.
      --driver-library-path       Extra library path entries to pass to the driver.
      --driver-class-path         Extra class path entries to pass to the driver. Note that
                                  jars added with --jars are automatically included in the
                                  classpath.

      --executor-memory MEM       Memory per executor (e.g. 1000M, 2G) (Default: 1G).

      --proxy-user NAME           User to impersonate when submitting the application.
                                  This argument does not work with --principal / --keytab.

      --help, -h                  Show this help message and exit.
      --verbose, -v               Print additional debug output.
      --version,                  Print the version of current Spark.

     Spark standalone with cluster deploy mode only:
      --driver-cores NUM          Cores for driver (Default: 1).

     Spark standalone or Mesos with cluster deploy mode only:
      --supervise                 If given, restarts the driver on failure.
      --kill SUBMISSION_ID        If given, kills the driver specified.
      --status SUBMISSION_ID      If given, requests the status of the driver specified.

     Spark standalone and Mesos only:
      --total-executor-cores NUM  Total cores for all executors.

     Spark standalone and YARN only:
      --executor-cores NUM        Number of cores per executor. (Default: 1 in YARN mode,
                                  or all available cores on the worker in standalone mode)

     YARN-only:
      --driver-cores NUM          Number of cores used by the driver, only in cluster mode
                                  (Default: 1).
      --queue QUEUE_NAME          The YARN queue to submit to (Default: "default").
      --num-executors NUM         Number of executors to launch (Default: 2).
                                  If dynamic allocation is enabled, the initial number of
                                  executors will be at least NUM.
      --archives ARCHIVES         Comma separated list of archives to be extracted into the
                                  working directory of each executor.
      --principal PRINCIPAL       Principal to be used to login to KDC, while running on
                                  secure HDFS.
      --keytab KEYTAB             The full path to the file that contains the keytab for the
                                  principal specified above. This keytab will be copied to
                                  the node running the Application Master via the Secure
                                  Distributed Cache, for renewing the login tickets and the
                                  delegation tokens periodically.